/*

    Timers.c  -  Routines for using software generated timers.

    Copyright (C) 2008  J. op den Brouw <jesse@opdenbrouw.nl>

    This file is part of Timers version 0.1

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

*/


#include <stdint.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include "Timers.h"

/* These are the timer data. They are defined static and therefore */
/* file scoped */
static volatile timer_t timers[NR_TIMERS];

/* Function:  timer_initialize_timers
 * Purpose:   initializes the (data structures of the) timers
 * Input:     none
 * Output:    none
 * Extra:     should be called before interrupts are freed.
 */
void timer_initialize_timers(void) {
	int8_t i;

	/* Loop through all timers */
	for (i=0; i<NR_TIMERS; i++) {
		/* Reset all values to zero */
		timers[i].inuse=0;
		timers[i].active=0;
		timers[i].ticks=0;
		timers[i].maxticks=0;
		timers[i].overflow=0;
	}		
}

/* Function:  timer_acquire_timer
 * Purpose:   tries to acquire a timer
 * Input:     desired timer number or -1 for automatic acquire
 * Output:    desired or first free timer number or -1 on failure
 */
int8_t timer_acquire_timer(int8_t nr) {
	int8_t i;

	/* Disable interrupts */
	cli();
	/* Test for timer number within bounds */
	if (nr>=0 && nr<NR_TIMERS) {
		/* Try to acquire requested timer */
		if (timers[nr].inuse==0) {
			timers[nr].inuse=1;
			timers[nr].active=0;
			timers[nr].ticks=0;
			timers[nr].maxticks=0;
			timers[nr].overflow=0;
		} else {
			nr=-1;
		}
	} else {
		/* Start searching for a free timer */
		nr=-1;
		for (i=0; i<NR_TIMERS; i++) {
			if (timers[i].inuse==0) {
				timers[i].inuse=1;
				timers[i].active=0;
				timers[i].ticks=0;
				timers[i].maxticks=0;
				timers[i].overflow=0;
				nr=i;
				break;
			}		
		}
	}
	/* Enable interrupts and return timer number */
	sei();
	return nr;
}

/* Function:  timer_release_timer
 * Purpose:   tries to release a timer
 * Input:     desired timer number
 * Output:    0 on succes or -1 on failure
 */
int8_t timer_release_timer(int8_t nr) {

	cli();
	if (nr>=0 && nr<NR_TIMERS) {
		if (timers[nr].inuse==1) {
			timers[nr].inuse=0;
			sei();
			return 0;
		}
	}
	sei();
	return -1;
}

/* Function:  timer_start_timer
 * Purpose:   starts a timer previously acquired
 * Input:     timer number
 * Output:    0 on succes, -1 on failure
 */
int8_t timer_start_timer(int8_t nr) {

	cli();
	if (nr>=0 && nr<NR_TIMERS) {
		if (timers[nr].inuse==1) {
			timers[nr].active=1;
			sei();
			return 0;
		}
	}
	sei();
	return -1;
}

/* Function:  timer_stop_timer
 * Purpose:   stops a timer previously started
 * Input:     timer number
 * Output:    0 on succes, -1 on failure
 */
int8_t timer_stop_timer(int8_t nr) {

	cli();
	if (nr>=0 && nr<NR_TIMERS) {
		if (timers[nr].active==1) {
			timers[nr].active=0;
			sei();
			return 0;
		}
	}
	sei();
	return -1;
}

/* Function:  timer_update_timers
 * Purpose:   updates the timer values and overflow values
 * Input:     none
 * Output:    none
 * Extra:     SHOULD ONLY BE CALLED FROM AN ISR, NEVER DIRECTLY
 */
void timer_update_timers(void) {
	uint8_t i;

	/* Loop through all timers */
	for (i=0; i<NR_TIMERS; i++) {
		/* Is timer running? */
		if (timers[i].active==1) {
			/* Decrease tick count */
			timers[i].ticks--;
			if (timers[i].ticks==0) {
				/* Hit bottom? Then reload timer and update overflow info */
				timers[i].ticks=timers[i].maxticks;
				timers[i].overflow++;
			}
		}
	}
}

/* Function:  timer_set_maxticks
 * Purpose:   sets the ticks to count
 * Input:     timer number, number of ticks
 * Output:    0 if succeeded, -1 on failure
 */
int8_t timer_set_maxticks(int8_t nr, timer_tick_t nr_ticks) {

	cli();
	/* Timer number within bounds? */
	if (nr>=0 && nr<NR_TIMERS) {
		/* Is timer in use? */
		if (timers[nr].inuse==1) {
			if (nr_ticks>0) {
				timers[nr].maxticks=nr_ticks;
				timers[nr].ticks=nr_ticks;
				sei();
				return 0;
			}
		}
	}
	sei();
	return -1;
}

/* Function:  timer_get_overflow
 * Purpose:   gets the overflow count
 * Input:     timer number
 * Output:    1 if overflow, 0 if no overflow, -1 on failure
 * Extra:     overflow count is decreased by one if there was an overflow
 */
int8_t timer_get_overflow(int8_t nr) {
	/* Default result value */
	int8_t result=-1;

	cli();
	/* Timer number within bounds? */
	if (nr>=0 && nr<NR_TIMERS) {
		/* Did timer overflow? */
		if (timers[nr].overflow>0) {
			/* Decrease overflow count and flag result */
			timers[nr].overflow--;
			result=1;
		} else {
			/* Nope, no overflow */
			result=0;
		}
	}
	sei();
	return result;
}
