/*

    Timers_test.c  -  Routines for testing the Timers routines.

    Copyright (C) 2008  J. op den Brouw <jesse@opdenbrouw.nl>

    This file is part of Timers version 0.1

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

*/

/* These routines are tested on an STK500 evaluation board with
 * Port B connected to the leds. The clock frequency is set to
 * 3.6864 MHz.
 */


/* Include some headers files */
#include <stdint.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include "Timers.h"


/* Now we calculate some values for T/C1 */
/* PRESCALER is the prescaler value, one of 1024, 256, etc. */
/* FREQ is the frequency at which compare match occurs */
/* LOAD is the calculated value for the compare match register */
#define PRESCALER 1024UL
#define FREQ 10UL
#define LOAD ((F_CPU/(PRESCALER*FREQ))-1)


/* Some counters */
volatile uint16_t cnt0=0, cnt1=0, cnt2=0, cnt3=0;

/* The ISR for T/C1 compare match interrupt */
ISR(TIMER1_COMPA_vect) {
	/* Update the timers. ONLY TO BE CALLED FROM AN ISR */
	timer_update_timers();
}


int main() {
	int8_t timerA, timerB, timerC, timerD, running=1;

	/* Set up Port B for output */
	DDRB=0xff;
	PORTB=0xff;

	/* Set up T/C1 */
	TCCR1A = (0<<WGM11)|(0<<WGM10);
	TCCR1B = (0<<WGM13)|(1<<WGM12)|(1<<CS12)|(0<<CS11)|(1<<CS10);
	OCR1A  = LOAD;
	TIMSK  = 1<<OCIE1A;

	/* Initialize Timers */
	timer_initialize_timers();

	/* Free interrupts */
    sei();

	/* Acquire three timers automaticly */
	timerA=timer_acquire_timer(-1);
	timerB=timer_acquire_timer(-1);
	timerC=timer_acquire_timer(-1);
	timerD=timer_acquire_timer(-1);

	/* Set the ticks */
	timer_set_maxticks(timerA, 1600);
	timer_set_maxticks(timerB, 9);
	timer_set_maxticks(timerC, 4);
	timer_set_maxticks(timerD, 11);

	/* Start the timers */
	timer_start_timer(timerA);
	timer_start_timer(timerB);
	timer_start_timer(timerC);
	timer_start_timer(timerD);

	/* Main loop; wait for timers to overflow */
	while (running) {
		if (timer_get_overflow(timerA)==1) {
			/* Break out of the while loop */
			cnt0++;
			running=0;
		}
		if (timer_get_overflow(timerB)==1) {
			/* Update counter 1 and toggle LED 7 */
			cnt1++;
			PORTB^=(1<<7);
		}
		if (timer_get_overflow(timerC)==1) {
			/* Update counter 2 and toggle LED 6 */
			cnt2++;
			PORTB^=(1<<6);
		}
		if (timer_get_overflow(timerD)==1) {
			/* Break out of the while(1) loop */
			PORTB^=(1<<5);
			cnt3++;
		}
	}
	
	/* Release the timers */
	timer_release_timer(timerA);
	timer_release_timer(timerB);
	timer_release_timer(timerC);
	timer_release_timer(timerD);


	
	return 0;
}
